var fs = require("fs");
var vm = require("vm");

function fireCallbacks(loader) {
    var callbacks = loader.callbacks || [];

    for (var i = 0, l = callbacks.length; i < l; ++i) {
        callbacks[i](loader.context);
    }
}

function loadScripts(loader) {
    if (!loader.isReady) {
        return;
    }

    if (loader.isLoaded) {
        return fireCallbacks(loader);
    }

    if (loader.loaded == loader.files.length) {
        for (var i = 0, l = loader.scripts.length; i < l; ++i) {
            vm.runInNewContext(loader.scripts[i], loader.context, loader.files[i]);
        }

        loader.isLoaded = true;
        fireCallbacks(loader);
    }
}

var globals = {
    setTimeout: setTimeout,
    clearTimeout: clearTimeout,
    setInterval: setInterval,
    clearInterval: clearInterval
};

module.exports = {
    added: 0,
    loaded: 0,

    create: function (context) {
        var loader = Object.create(this);
        loader.context = loader.initContext(context);
        loader.scripts = [];
        loader.files = [];

        return loader;
    },

    onLoad: function (loader) {
        this.callbacks = this.callbacks || [];
        this.callbacks.push(loader);
    },

    add: function (script) {
        var i = this.files.length;
        this.files.push(script);
        var loader = this;

        fs.readFile(script, "utf-8", function (err, data) {
            loader.scripts[i] = data;
            loader.loaded += 1;
            loadScripts(loader);
        });
    },

    addScript: function (script) {
        this.files.push("");
        this.scripts.push(script);
        this.loaded += 1;
        loadScripts(this);
    },

    load: function () {
        this.isReady = true;

        if (this.loaded) {
            loadScripts(this);
        }
    },

    initContext: function (context) {
        context = context || {};

        for (var prop in globals) {
            if (!context[prop]) {
                context[prop] = globals[prop];
            }
        }

        return context;
    }
};
